import {Component, AfterContentInit, Input, Output, EventEmitter} from '@angular/core';
import {Http, HTTP_PROVIDERS} from '@angular/http';
import 'rxjs/add/operator/map';

// global variables for imported js libraries
declare var Ext:any;
declare var Sch:any;

@Component({
    selector: 'scheduler',
    template: '<div id="scheduler"></div>',
    providers: [HTTP_PROVIDERS]
})

export class SchedulerComponent implements AfterContentInit {

    @Input() title:string;
    @Input() rowHeight:number;
    @Input() split:boolean;
    @Input() preset:string;
    @Input() url:string;
    @Input() startDate:string;
    @Input() endDate:string;

    @Output() onEventSelect = new EventEmitter<any>();
    @Output() eventCount:number;

    scheduler:any;
    resourceStore:any;
    eventStore:any;

    constructor(private http:Http) { }

    ngAfterContentInit() {
        // parse dates
        var start = new Date(this.startDate),
            end   = new Date(this.endDate);

        this.eventStore = Ext.create('Sch.data.EventStore', {
            model: 'Sch.model.Event',
            proxy: {
                type: 'memory'
            }
        });

        this.resourceStore = Ext.create('Sch.data.ResourceStore')

        // create scheduler
        this.scheduler = Ext.create('Sch.panel.SchedulerGrid', {
            // input configs (from tag)
            rowHeight: this.rowHeight || 35,
            title: this.title,
            split: this.split,
            viewPreset: this.preset,
            startDate: start,
            endDate: end,

            barMargin: 2,
            renderTo: 'scheduler',
            bodyBorder: false,
            columns: [
                {header: 'Name', sortable: true, width: 160, dataIndex: 'Name'}
            ],
            resourceStore: this.resourceStore,
            eventStore: this.eventStore,
            listeners: {
                eventselect: function(em, eventRecord) {
                    this.onEventSelect.emit(eventRecord);
                },
                scope: this
            }
        });

        // load data
        this.http.get(this.url)
            .map(r => r.json())
            .subscribe(
                data => this.loadData(data),
                err => console.error(err)
            );
    }

    /**
     * Consumes json in format { resources: [], events: [] }
     * @param data
     */
    loadData(data):void {
        this.resourceStore.loadData(data.resources);
        this.eventStore.proxy.data = data.events;
        this.eventStore.load();
        this.eventCount = this.eventStore.getCount();
    }

    /**
     * Add event to random resource on random day
     */
    addRandomEvent():void {
        var eventStore = this.eventStore,
            startDay   = (Math.floor(Math.random() * 7) + 5);
        eventStore.add({
            ResourceId: 'r' + (Math.floor(Math.random() * 6) + 1),
            Name: 'New event ' + eventStore.getCount(),
            StartDate: '2016-09-' + Ext.String.leftPad(startDay, 2, '0'),
            EndDate: '2016-09-' + Ext.String.leftPad((startDay + 1), 2, '0')
        });
        this.eventCount = eventStore.getCount();
    }
}
